//Datei: DS1821.C
// Hauptprogramm
//
// Przisionsthermometer
// mit PIC16F84 und DS1821
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 24.04.2001
// Compiler PicLite

// Verwendete LCD-Anzeige 2x16 Zeichen oder 2x8 Zeichen

#include "prozess.h"      /* Der verwendete Prozessor */

#include "ds1821.h"
#include "protos.h"
#include <math.h>

 __CONFIG(0x3FF1); //Configuration Word

//Globale Flags und Rckgabevariablen

char sign;       //Vorzeichen und 7Bit Temperaturwert
int temp;     //Temperatur
int countrem; //9 Bit Register 1
int countdeg; //9 Bit Register 2
int tempbuf;  //bergabevariable fr Anzeige
int tempmin;  //Kleinste gemessene Temperatur
int tempmax;  //grte gemessene Temperatur


/* Hauptprogramm */
void main()
{
 unsigned char i;
 
 InitPIC(); //Interrupts und so weiter
 LCDInit(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen

 MasterReset();
 SendByte(WRITESTATUS);
 SendByte(0x41); //One Shot Operation einstellen
 Delay1ms(255);  //Etwas warten

 tempmin=12500;
 tempmax=-5500;
 
 while(1) // Anfang Programm Endlosschleife
  {

   MasterReset();      //Messung starten
   SendByte(STARTCONV);

  //Hier Status Polling fr Ende der Messung
   PollStatus();
   
   MasterReset();
   SendByte(READTEMP);
   ReadByte(8);       //Temperatur lesen
   sign=(tempbuf&0x00FF); //Nur die unteren 8 Bits nehmen !
   
   //Die beiden 9 Bit Register lesen
   MasterReset();
   SendByte(READCOUNT);
   ReadByte(9);       //Remain lesen
   countrem=tempbuf;

   MasterReset();
   SendByte(LOADREM); //Zweites Register laden

   MasterReset();
   SendByte(READCOUNT);
   ReadByte(9);       //Degree lesen
   countdeg=tempbuf;
   
//Zum testen der Anzeige:
//   sign=-54;   // Ergebnis = -54.09
//   sign=54;   // Ergebnis = +53.91
//   sign=0;     // Ergebnis = -0.09
//   countrem=(int)300; 
//   countdeg=(int)512;

   //signed integer aus Zweierkomplement machen wenn negativ
   if(sign<0)
    {
     sign=sign+255;     //Einerkomplement
     sign++;            //Zweierkomplement
    }

   temp=(int)sign; //char zu int

   //Temperatur * 100  => -5500 bis 12500
   //Schiebt den Wert um zwei Dezimalstellen nach links
   //So spare ich mir unten eine Subtraktion und eine Addition
   //im Floating Point Format
   tempbuf=temp;
   for(i=0; i<99; i++) temp+=tempbuf; //Die einfache Art * 100 zu nehmen


//Wenn countdeg=0 mte durch 0 geteilt werden. Abfangen
   if(countdeg>0) //Schade jetzt mu ich rechnen
    {
     //countdeg-countrem in integer rechnen Ergebnis in countrem
     countrem=countdeg-countrem;
     //Das Ergebnis ist immer positiv

     //Ich mchte zwei Stellen hinter dem Komma haben
     //Das Ergebnis der Division wre aber kleiner 1
     //Um wieder FloatingPoint Operation zu vermeiden
     //nehme ich den Zhler * 100
     tempbuf=countrem;
     for(i=0; i<99; i++) countrem+=tempbuf; //Die einfache Art * 100 zu nehmen

     countrem=countrem/countdeg; //Und teilen
    } 
   else //countdeg war 0, dann 100 * 1/2 LSB(=0.5) addieren
    { countrem=50; } //1/2 LSB
   
   temp+=countrem; //Genaue Temperatur ausrechnen
   //LSB = 0.5 * 100 = 50
   //Temperatur - LSB ausrechnen
   temp-=50;

   //Temperatur fnfstellig anzeigen
   tempbuf=temp; //Ergebnis in tempbuf bergeben
   LCDPos(1,1);
   ShowDisplay();

   if(tempmin>temp) tempmin=temp; //Neue Temperatur ist kleiner
   if(tempmax<temp) tempmax=temp; //Neue Temperatur ist grer

   //Je nach Zustand am PORT Temperatur in zweiter Zeile anzeigen
   if(MINMAX==0) tempbuf=tempmin;
   else           tempbuf=tempmax;
   LCDPos(2,1);
   ShowDisplay(); //Min oder Maxtemp anzeigen

   if(CLEAR==0) //Min.Max zurcksetzen
    {
     tempmin=12500;
     tempmax=-5500;
    } 

  } // Ende Programm Endlosschleife
}

//************************************************
// Zeigt die 5 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 if(tempbuf<0) //Falls negativ, Absolutwert anzeigen
  {
   tempbuf=-tempbuf;
   LCDWriteByte('-'); //Vorzeichen - auf Display
  }
 else
  {
   LCDWriteByte('+'); //Vorzeichen + auf Display
  } 

 B1=(unsigned char)(tempbuf>>8); //Highbyte
 B0=(unsigned char)(tempbuf&0xFF); //Lowbyte

 BinToBCD16(); //unsigned int zu BCD

 LCDWriteByte(R0+0x30); //Zehntausender
 ShowBCD(R1); //Tausender Hunderter
 LCDWriteByte('.'); //"." anzeigen
 ShowBCD(R2); //Zehner Einer
 LCDWriteByte(0xDF); //"" Einheit anzeigen

}
